@@ -6,6 +6,9 @@
  */
 
 #include "syshdrs.h"
+#ifdef PRAGMA_HDRSTOP
+#	pragma hdrstop
+#endif
 #include "shell.h"
 #include "trace.h"
 #include "util.h"
@@ -16,6 +19,7 @@ char gHome[256];
 char gShell[256];
 char gOurDirectoryPath[260];
 char gOurInstallationPath[260];
+int gNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = 0;
 #ifdef ncftp
 static int gResolveSig;
 #endif
@@ -182,60 +186,20 @@ OutOfMemory(void)
 
 
 void
-MyInetAddr(char *dst, size_t siz, char **src, int i)
+MyInetAddr(char *dst, size_t siz, void *src, int i)
 {
 	struct in_addr *ia;
-#ifndef HAVE_INET_NTOP
-	char *cp;
-#endif
 
 	(void) Strncpy(dst, "???", siz);
-	if (src != (char **) 0) {
-		ia = (struct in_addr *) src[i];
-#ifdef HAVE_INET_NTOP	/* Mostly to workaround bug in IRIX 6.5's inet_ntoa */
-		(void) inet_ntop(AF_INET, ia, dst, siz - 1);
-#else
-		cp = inet_ntoa(*ia);
-		if ((cp != (char *) 0) && (cp != (char *) -1) && (cp[0] != '\0'))
-			(void) Strncpy(dst, cp, siz);
-#endif
+	if (src != NULL) {
+		ia = ((struct in_addr **) src)[i];
+		InetNtoA(dst, ia, siz);
 	}
 }	/* MyInetAddr */
 
 
 
 
-/* On entry, you should have 'host' be set to a symbolic name (like
- * cse.unl.edu), or set to a numeric address (like 129.93.3.1).
- * If the function fails, it will return NULL, but if the host was
- * a numeric style address, you'll have the ip_address to fall back on.
- */
-
-struct hostent *
-GetHostEntry(const char *host, struct in_addr *ip_address)
-{
-	struct in_addr ip;
-	struct hostent *hp;
-	
-	/* See if the host was given in the dotted IP format, like "36.44.0.2."
-	 * If it was, inet_addr will convert that to a 32-bit binary value;
-	 * it not, inet_addr will return (-1L).
-	 */
-	ip.s_addr = inet_addr(host);
-	if (ip.s_addr != INADDR_NONE) {
-		hp = gethostbyaddr((char *) &ip, (int) sizeof(ip), AF_INET);
-	} else {
-		/* No IP address, so it must be a hostname, like ftp.wustl.edu. */
-		hp = gethostbyname(host);
-		if (hp != NULL)
-			ip = * (struct in_addr *) hp->h_addr_list;
-	}
-	if (ip_address != NULL)
-		*ip_address = ip;
-	return (hp);
-}	/* GetHostEntry */
-
-
 
 /* This simplifies a pathname, by converting it to the
  * equivalent of "cd $dir ; dir=`pwd`".  In other words,
@@ -243,6 +207,8 @@ GetHostEntry(const char *host, struct in_addr *ip_address)
  * "$PWD/../tmp////./../xx/", it would be converted to
  * "/usr/spool/xx".
  */
+
+
 void
 CompressPath(char *const dst, const char *const src, const size_t dsize)
 {
@@ -251,6 +217,7 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 	char *d, *lim;
 	char *a, *b;
 
+#define isslash(c) ((c == '/') || (c == '\\'))
 	if (src[0] == '\0') {
 		*dst = '\0';
 		return;
@@ -262,9 +229,9 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 	for (;;) {
 		c = *s;
 		if (c == '.') {
-			if (((s == src) || (s[-1] == '/')) && ((s[1] == '/') || (s[1] == '\0'))) {
+			if (((s == src) || isslash(s[-1])) && (isslash(s[1]) || (s[1] == '\0'))) {
 				/* Don't copy "./" */
-				if (s[1] == '/')
+				if (isslash(s[1]))
 					++s;
 				++s;
 			} else if (d < lim) {
@@ -272,7 +239,7 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 			} else {
 				++s;
 			}
-		} else if (c == '/') {
+		} else if (isslash(c)) {
 			/* Don't copy multiple slashes. */
 			if (d < lim)
 				*d++ = *s++;
@@ -280,12 +247,12 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 				++s;
 			for (;;) {
 				c = *s;
-				if (c == '/') {
+				if (isslash(c)) {
 					/* Don't copy multiple slashes. */
 					++s;
 				} else if (c == '.') {
 					c = s[1];
-					if (c == '/') {
+					if (isslash(c)) {
 						/* Skip "./" */
 						s += 2;
 					} else if (c == '\0') {
@@ -300,7 +267,7 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 			}
 		} else if (c == '\0') {
 			/* Remove trailing slash. */
-			if ((d[-1] == '/') && (d > (dst + 1)))
+			if (isslash(d[-1]) && (d > (dst + 1)))
 				d[-1] = '\0';
 			*d = '\0';
 			break;
@@ -323,14 +290,14 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 			/* Get the next node in the path. */
 			if (*a == '\0')
 				return;
-			if (*a == '/') {
+			if (isslash(*a)) {
 				++a;
 				break;
 			}
 			++a;
 		}
 		if ((b[0] == '.') && (b[1] == '.')) {
-			if (b[2] == '/') {
+			if (isslash(b[2])) {
 				/* We don't know what the parent of this
 				 * node would be.
 				 */
@@ -338,16 +305,16 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 			}
 		}
 		if ((a[0] == '.') && (a[1] == '.')) {
-			if (a[2] == '/') {
+			if (isslash(a[2])) {
 				/* Remove the .. node and the one before it. */
-				if ((b == dst) && (*dst == '/'))
+				if ((b == dst) && (isslash(*dst)))
 					(void) memmove(b + 1, a + 3, strlen(a + 3) + 1);
 				else
 					(void) memmove(b, a + 3, strlen(a + 3) + 1);
 				a = dst;	/* Start over. */
 			} else if (a[2] == '\0') {
 				/* Remove a trailing .. like:  /aaa/bbb/.. */
-				if ((b <= dst + 1) && (*dst == '/'))
+				if ((b <= dst + 1) && isslash(*dst))
 					dst[1] = '\0';
 				else
 					b[-1] = '\0';
@@ -360,24 +327,39 @@ CompressPath(char *const dst, const char *const src, const size_t dsize)
 			}
 		}
 	}
+#undef isslash
 }	/* CompressPath */
 
 
 
 void
-PathCat(char *const dst, const size_t dsize, const char *const cwd, const char *const src)
+PathCat(char *const dst, const size_t dsize, const char *const cwd, const char *const src, int dosCompat)
 {
 	char *cp;
 	char tmp[512];
 
-	if (src[0] == '/') {
+	if (dosCompat != 0) {
+		if ((isalpha((int) cwd[0])) && (cwd[1] == ':')) {
+			/* A new fully-qualified DOS path was requested. */
+			CompressPath(dst, src, dsize);
+			return;
+		}
+	} else if (src[0] == '/') {
+		/* A new fully-qualified UNIX path was requested. */
 		CompressPath(dst, src, dsize);
 		return;
 	}
-	cp = Strnpcpy(tmp, (char *) cwd, sizeof(tmp) - 1);
-	*cp++ = '/';
+	cp = Strnpcpy(tmp, cwd, sizeof(tmp) - 1);
+	if (dosCompat) {
+		if ((dst[1] != ':') || (dst[2] == '/'))
+			*cp++ = '/';
+		else
+			*cp++ = '\\';
+	} else {
+		*cp++ = '/';
+	}
 	*cp = '\0';
-	(void) Strnpcat(cp, (char *) src, sizeof(tmp) - (cp - tmp));
+	(void) Strnpcat(cp, src, sizeof(tmp) - (cp - tmp));
 	CompressPath(dst, tmp, dsize);
 }	/* PathCat */
 
@@ -414,7 +396,7 @@ FileToURL(char *url, size_t urlsize, const char *const fn, const char *const rcw
 	ulen = strlen(url);
 	dst = url + ulen;
 	dsize = urlsize - ulen;
-	PathCat(dst, dsize, rcwd, fn);
+	PathCat(dst, dsize, rcwd, fn, 0);
 	if ((startdir != NULL) && (startdir[0] != '\0') && (startdir[1] /* i.e. not "/" */ != '\0')) {
 		if (strncmp(dst, startdir, strlen(startdir)) == 0) {
 			/* Form relative URL. */
@@ -457,14 +439,14 @@ AbbrevStr(char *dst, const char *src, size_t max, int mode)
 		if (mode == 0) {
 			/* ...Put ellipses at left */
 			(void) strcpy(dst, "...");
-			(void) Strncat(dst, (char *) src + len - (int) max + 3, max + 1);
+			(void) Strncat(dst, src + len - (int) max + 3, max + 1);
 		} else {
 			/* Put ellipses at right... */
-			(void) Strncpy(dst, (char *) src, max + 1);
+			(void) Strncpy(dst, src, max + 1);
 			(void) strcpy(dst + max - 3, "...");
 		}
 	} else {
-		(void) Strncpy(dst, (char *) src, max + 1);
+		(void) Strncpy(dst, src, max + 1);
 	}
 }	/* AbbrevStr */
 
@@ -573,6 +555,8 @@ InitOurDirectory(void)
 			cp = strrchr(gOurDirectoryPath, '"');
 			if ((cp != NULL) && (cp[1] == '\0'))
 				*cp = '\0';
+			(void) STRNCAT(gOurDirectoryPath, "\\");
+			(void) STRNCAT(gOurDirectoryPath, kOurDirectoryName);
 		} else {
 			STRNCPY(gOurDirectoryPath, gOurInstallationPath);
 			if (gUser[0] == '\0') {
@@ -756,7 +740,6 @@ StrToBool(const char *const s)
 	result = 0;
 	switch (c) {
 		case 'f':			       /* false */
-			/*FALLTHROUGH*/
 		case 'n':			       /* no */
 			break;
 		case 'o':			       /* test for "off" and "on" */
@@ -765,9 +748,9 @@ StrToBool(const char *const s)
 				c = tolower(c);
 			if (c == 'f')
 				break;
-			/*FALLTHROUGH*/
+			result = 1;
+			break;
 		case 't':			       /* true */
-			/*FALLTHROUGH*/
 		case 'y':			       /* yes */
 			result = 1;
 			break;
@@ -823,7 +806,7 @@ CancelGetHostByName(int sigNum)
 
 
 int
-GetHostByName(char *const volatile dst, size_t dsize, const char *const hn, int t)
+MyGetHostByName(char *const volatile dst, size_t dsize, const char *const hn, int t)
 {
 #if defined(WIN32) || defined(_WINDOWS)
 	struct hostent *hp;
@@ -838,7 +821,7 @@ GetHostByName(char *const volatile dst, size_t dsize, const char *const hn, int
 	hp = gethostbyname(hn);
 	if (hp != NULL) {
 		(void) memcpy(&ina.s_addr, hp->h_addr_list[0], (size_t) hp->h_length);
-		(void) Strncpy(dst, inet_ntoa(ina), dsize);
+		InetNtoA(dst, ((struct in_addr **) hp->h_addr_list)[0], dsize);
 		return (0);
 	}
 
@@ -846,7 +829,7 @@ GetHostByName(char *const volatile dst, size_t dsize, const char *const hn, int
 	int sj;
 	vsigproc_t osigpipe, osigint, osigalrm;
 	struct hostent *hp;
-#ifndef HAVE_INET_NTOP
+#ifdef HAVE_INET_ATON
 	struct in_addr ina;
 #endif
 
@@ -894,13 +877,7 @@ GetHostByName(char *const volatile dst, size_t dsize, const char *const hn, int
 		(void) NcSignal(SIGINT, osigint);
 		(void) NcSignal(SIGALRM, osigalrm);
 		if (hp != NULL) {
-#ifdef HAVE_INET_NTOP	/* Mostly to workaround bug in IRIX 6.5's inet_ntoa */
-			(void) memset(dst, 0, dsize);
-			(void) inet_ntop(AF_INET, hp->h_addr_list[0], dst, dsize - 1);
-#else
-			(void) memcpy(&ina.s_addr, hp->h_addr_list[0], (size_t) hp->h_length);
-			(void) Strncpy(dst, inet_ntoa(ina), dsize);
-#endif
+			InetNtoA(dst, ((struct in_addr **) hp->h_addr_list)[0], dsize);
 			return (0);
 		}
 	}
@@ -908,7 +885,7 @@ GetHostByName(char *const volatile dst, size_t dsize, const char *const hn, int
 
 	*dst = '\0';
 	return (-1);
-}	/* GetHostByName */
+}	/* MyGetHostByName */
 
 
 
@@ -919,19 +896,10 @@ time_t UnDate(char *dstr)
 #ifndef HAVE_MKTIME
 	return ((time_t) -1);
 #else
-	struct tm ut, *t;
-	time_t now;
+	struct tm ut;
 	time_t result = (time_t) -1;
 
-	(void) time(&now);
-	t = localtime(&now);
-
-	/* Copy the whole structure of the 'tm' pointed to by t, so it will
-	 * also set all fields we don't specify explicitly to be the same as
-	 * they were in t.  That way we copy non-standard fields such as
-	 * tm_gmtoff, if it exists or not.
-	 */
-	ut = *t;
+	(void) Localtime(0, &ut);
 
 	/* The time we get back from the server is (should be) in UTC. */
 	if (sscanf(dstr, "%04d%02d%02d%02d%02d%02d",
@@ -1132,7 +1100,7 @@ void SysPerror(const char *const errMsg)
 		NULL
 		);
 
-	if (reason[strlen(reason) - 1] = '\n')
+	if (reason[strlen(reason) - 1] == '\n')
 		reason[strlen(reason) - 1] = '\0';
 	(void) fprintf(stderr, "%s: %s\n", errMsg, reason);
 }	/* SysPerror */
