@@ -1,6 +1,6 @@
 /* ncftpput.c
  *
- * Copyright (c) 1996-2000 Mike Gleason, NCEMRSoft.
+ * Copyright (c) 1996-2001 Mike Gleason, NCEMRSoft.
  * All rights reserved.
  *
  * A simple, non-interactive utility to send files to a remote FTP server.
@@ -8,28 +8,24 @@
  */
 
 #include "syshdrs.h"
+#ifdef PRAGMA_HDRSTOP
+#	pragma hdrstop
+#endif
 
 #if defined(WIN32) || defined(_WINDOWS)
-#	include "..\ncftp\getopt.h"
-#	define getopt Getopt
-#	define optarg gOptArg
-#	define optind gOptInd
-	WSADATA wsaData;
-	int wsaInit = 0;
-
-	__inline void DisposeWinsock(int aUNUSED) { if (wsaInit > 0) WSACleanup(); wsaInit--; }
 #	include "..\ncftp\util.h"
 #	include "..\ncftp\spool.h"
 #	include "..\ncftp\pref.h"
+#	include "..\ncftp\getline.h"
 #else
 #	include "../ncftp/util.h"
 #	include "../ncftp/spool.h"
 #	include "../ncftp/pref.h"
+#	include "../ncftp/getline.h"
 #endif
 
 #include "gpshare.h"
 
-
 FTPLibraryInfo gLib;
 FTPConnectionInfo fi;
 
@@ -40,10 +36,7 @@ extern char gFirewallPass[32];
 extern unsigned int gFirewallPort;
 extern char gFirewallExceptionList[256];
 extern int gFwDataPortMode;
-extern char gOS[], gVersion[];
-
-extern char *optarg;
-extern int optind;
+extern const char gOS[], gVersion[];
 
 static void
 Usage(void)
@@ -73,17 +66,22 @@ Usage(void)
 	(void) fprintf(fp, "\
   -c     Use stdin as input file to write on remote host.\n\
   -A     Append to remote files instead of overwriting them.\n\
-  -z/-Z  Do (do not) not try to resume uploads (default: -Z).\n\
+  -z/-Z  Do (do not) try to resume uploads (default: -Z).\n\
   -T XX  Upload into temporary files prefixed by XX.\n");
 	(void) fprintf(fp, "\
   -S XX  Upload into temporary files suffixed by XX.\n\
   -DD    Delete local file after successfully uploading it.\n\
-  -b     Run in background (submit job to \"ncftpbatch\").\n\
+  -b     Run in background (submit job to \"ncftpbatch\" and run).\n\
+  -bb    Same as \"-b\" but queue only (do not run \"ncftpbatch\").\n\
   -E     Use regular (PORT) data connections.\n\
   -F     Use passive (PASV) data connections (default).\n\
-  -y     Try using \"SITE UTIME\" to preserve timestamps on remote host.\n\
+  -y     Try using \"SITE UTIME\" to preserve timestamps on remote host.\n");
+	(void) fprintf(fp, "\
   -B XX  Try setting the SO_SNDBUF size to XX.\n\
   -r XX  Redial XX times until connected.\n\
+  -W XX  Send raw FTP command XX after logging in.\n\
+  -X XX  Send raw FTP command XX after each file transferred.\n\
+  -Y XX  Send raw FTP command XX before logging out.\n\
   -R     Recursive mode; copy whole directory trees.\n");
 	(void) fprintf(fp, "\nExamples:\n\
   ncftpput -u gleason -p my.password Elwood.probe.net /home/gleason stuff.txt\n\
@@ -95,7 +93,7 @@ Usage(void)
 	(void) fprintf(fp, "This was built using LibNcFTP (http://www.ncftp.com/libncftp).\n");
 
 	ClosePager(fp);
-	DisposeWinsock(0);
+	DisposeWinsock();
 	exit(kExitUsage);
 }	/* Usage */
 
@@ -103,9 +101,9 @@ Usage(void)
 
 
 static void
-Abort(int sigNumUNUSED)
+Abort(int sigNum)
 {
-	signal(SIGINT, Abort);
+	signal(sigNum, Abort);
 
 	/* Hopefully the I/O operation in progress
 	 * will complete, and we'll abort before
@@ -118,18 +116,18 @@ Abort(int sigNumUNUSED)
 	 * next ^C abends the program.
 	 */
 	if (fi.cancelXfer >= 2)
-		signal(SIGINT, SIG_DFL);
+		signal(sigNum, SIG_DFL);
 }	/* Abort */
 
 
 
 
 static int
-Copy(FTPCIPtr cip, char *dstdir, char **files, int rflag, int xtype, int appendflag, const char *tmppfx, const char *tmpsfx, int resumeflag, int deleteflag)
+Copy(FTPCIPtr cip, const char *const dstdir, char **const files, const int rflag, const int xtype, const int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const char *const perfilecmd)
 {
 	int i;
 	int result;
-	char *file;
+	const char *file;
 	int rc = 0;
 
 	for (i=0; ; i++) {
@@ -142,10 +140,12 @@ Copy(FTPCIPtr cip, char *dstdir, char **files, int rflag, int xtype, int appendf
 #else
 			kGlobNo,	/* Shell does the glob for you */
 #endif
-			xtype, appendflag, tmppfx, tmpsfx, resumeflag, deleteflag, NoConfirmResumeUploadProc, 0);
+			xtype, appendflag, tmppfx, tmpsfx, resumeflag, deleteflag, kNoFTPConfirmResumeUploadProc, 0);
 		if (result != 0) {
 			FTPPerror(cip, result, kErrCouldNotStartDataTransfer, "ncftpput", file);
 			rc = result;
+		} else {
+			(void) AdditionalCmd(cip, perfilecmd, file);
 		}
 	}
 	return (rc);
@@ -153,7 +153,7 @@ Copy(FTPCIPtr cip, char *dstdir, char **files, int rflag, int xtype, int appendf
 
 
 
-int
+main_void_return_t
 main(int argc, char **argv)
 {
 	int result, c;
@@ -168,7 +168,6 @@ main(int argc, char **argv)
 	int wantMkdir = 0;
 	char *Umask = NULL;
 	char *dstdir = NULL;
-	const char *errstr;
 	char **files = (char **) 0;
 	int progmeters;
 	int usingcfg = 0;
@@ -181,22 +180,30 @@ main(int argc, char **argv)
 	char *ufilep;
 	const char *udirp;
 	char ufile[256];
-	char *password;
+	char precmd[320], postcmd[320], perfilecmd[320];
+	GetoptInfo opt;
 
 	InitWinsock();
+#if (defined(SOCKS)) && (SOCKS >= 5)
+	SOCKSinit(argv[0]);
+#endif	/* SOCKS */
+#ifdef SIGPOLL
+	NcSignal(SIGPOLL, (FTPSigProc) SIG_IGN);
+#endif
 	result = FTPInitLibrary(&gLib);
 	if (result < 0) {
 		(void) fprintf(stderr, "ncftpput: init library error %d (%s).\n", result, FTPStrError(result));
-		DisposeWinsock(0);
+		DisposeWinsock();
 		exit(kExitInitLibraryFailed);
 	}
 	result = FTPInitConnectionInfo(&gLib, &fi, kDefaultFTPBufSize);
 	if (result < 0) {
 		(void) fprintf(stderr, "ncftpput: init connection info error %d (%s).\n", result, FTPStrError(result));
-		DisposeWinsock(0);
+		DisposeWinsock();
 		exit(kExitInitConnInfoFailed);
 	}
 
+	InitUserInfo();
 	fi.dataPortMode = kFallBackToSendPortMode;
 	LoadFirewallPrefs(0);
 	if (gFwDataPortMode >= 0)
@@ -208,49 +215,56 @@ main(int argc, char **argv)
 	fi.errLog = stderr;
 	(void) STRNCPY(fi.user, "anonymous");
 	progmeters = GetDefaultProgressMeterSetting();
+	precmd[0] = '\0';
+	postcmd[0] = '\0';
+	perfilecmd[0] = '\0';
 
-	while ((c = getopt(argc, argv, "P:u:j:p:e:d:U:t:mar:RvVf:AT:S:EFcyZzDbB:")) > 0) switch(c) {
+	GetoptReset(&opt);
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:U:t:mar:RvVf:AT:S:EFcyZzDbB:W:X:Y:")) > 0) switch(c) {
 		case 'P':
-			fi.port = atoi(optarg);	
+			fi.port = atoi(opt.arg);	
 			break;
 		case 'u':
-			(void) STRNCPY(fi.user, optarg);
+			(void) STRNCPY(fi.user, opt.arg);
+			memset(opt.arg, '*', strlen(fi.user));
 			break;
 		case 'j':
-			(void) STRNCPY(fi.acct, optarg);
+			(void) STRNCPY(fi.acct, opt.arg);
+			memset(opt.arg, '*', strlen(fi.acct));
 			break;
 		case 'p':
-			(void) STRNCPY(fi.pass, optarg);	/* Don't recommend doing this! */
+			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
+			memset(opt.arg, '*', strlen(fi.pass));
 			break;
 		case 'e':
-			if (strcmp(optarg, "stdout") == 0)
+			if (strcmp(opt.arg, "stdout") == 0)
 				fi.errLog = stdout;
-			else if (optarg[0] == '-')
+			else if (opt.arg[0] == '-')
 				fi.errLog = stdout;
-			else if (strcmp(optarg, "stderr") == 0)
+			else if (strcmp(opt.arg, "stderr") == 0)
 				fi.errLog = stderr;
 			else
-				fi.errLog = fopen(optarg, FOPEN_APPEND_TEXT);
+				fi.errLog = fopen(opt.arg, FOPEN_APPEND_TEXT);
 			break;
 		case 'D':
 			/* Require two -D's in case they typo. */
 			nD++;
 			break;
 		case 'd':
-			if (strcmp(optarg, "stdout") == 0)
+			if (strcmp(opt.arg, "stdout") == 0)
 				fi.debugLog = stdout;
-			else if (optarg[0] == '-')
+			else if (opt.arg[0] == '-')
 				fi.debugLog = stdout;
-			else if (strcmp(optarg, "stderr") == 0)
+			else if (strcmp(opt.arg, "stderr") == 0)
 				fi.debugLog = stderr;
 			else
-				fi.debugLog = fopen(optarg, FOPEN_APPEND_TEXT);
+				fi.debugLog = fopen(opt.arg, FOPEN_APPEND_TEXT);
 			break;
 		case 'U':
-			Umask = optarg;
+			Umask = opt.arg;
 			break;
 		case 't':
-			SetTimeouts(&fi, optarg);
+			SetTimeouts(&fi, opt.arg);
 			break;
 		case 'm':
 			wantMkdir = 1;
@@ -259,7 +273,7 @@ main(int argc, char **argv)
 			xtype = kTypeAscii;	/* Use ascii. */
 			break;
 		case 'r':
-			SetRedial(&fi, optarg);
+			SetRedial(&fi, opt.arg);
 			break;
 		case 'R':
 			rflag = 1;
@@ -271,17 +285,17 @@ main(int argc, char **argv)
 			progmeters = 0;
 			break;
 		case 'f':
-			ReadConfigFile(optarg, &fi);
+			ReadConfigFile(opt.arg, &fi);
 			usingcfg = 1;
 			break;
 		case 'A':
 			appendflag = 1;
 			break;
 		case 'T':
-			tmppfx = optarg;
+			tmppfx = opt.arg;
 			break;
 		case 'S':
-			tmpsfx = optarg;
+			tmpsfx = opt.arg;
 			break;
 		case 'E':
 			fi.dataPortMode = kSendPortMode;
@@ -305,46 +319,51 @@ main(int argc, char **argv)
 			batchmode++;
 			break;
 		case 'B':
-			fi.dataSocketSBufSize = (size_t) atol(optarg);	
+			fi.dataSocketSBufSize = (size_t) atol(opt.arg);	
+			break;
+		case 'W':
+			STRNCAT(precmd, opt.arg);
+			STRNCAT(precmd, "\n");
+			break;
+		case 'X':
+			STRNCAT(perfilecmd, opt.arg);
+			STRNCAT(perfilecmd, "\n");
+			break;
+		case 'Y':
+			STRNCAT(postcmd, opt.arg);
+			STRNCAT(postcmd, "\n");
 			break;
 		default:
 			Usage();
 	}
 	if (usingcfg != 0) {
 		if (ftpcat == 0) {
-			if (optind > argc - 2)
+			if (opt.ind > argc - 2)
 				Usage();
-			dstdir = argv[optind + 0];
-			files = argv + optind + 1;
+			dstdir = argv[opt.ind + 0];
+			files = argv + opt.ind + 1;
 		} else {
-			if (optind > argc - 2)
+			if (opt.ind > argc - 2)
 				Usage();
-			(void) STRNCPY(fi.host, argv[optind]);
+			(void) STRNCPY(fi.host, argv[opt.ind]);
 		}
 	} else {
 		if (ftpcat == 0) {
-			if (optind > argc - 3)
+			if (opt.ind > argc - 3)
 				Usage();
-			(void) STRNCPY(fi.host, argv[optind]);
-			dstdir = argv[optind + 1];
-			files = argv + optind + 2;
+			(void) STRNCPY(fi.host, argv[opt.ind]);
+			dstdir = argv[opt.ind + 1];
+			files = argv + opt.ind + 2;
 		} else {
-			if (optind > argc - 2)
+			if (opt.ind > argc - 2)
 				Usage();
-			(void) STRNCPY(fi.host, argv[optind]);
+			(void) STRNCPY(fi.host, argv[opt.ind]);
 		}
 	}
 
-	InitUserInfo();
-
 	if (strcmp(fi.user, "anonymous") && strcmp(fi.user, "ftp")) {
 		if (fi.pass[0] == '\0') {
-			password = GetPass2("Password: ");		
-			if (password != NULL) {
-				(void) STRNCPY(fi.pass, password);
-				/* Don't leave cleartext password in memory. */
-				(void) memset(password, 0, strlen(fi.pass));
-			}
+			(void) gl_getpass("Password: ", fi.pass, sizeof(fi.pass));
 		}
 	}
 
@@ -364,6 +383,9 @@ main(int argc, char **argv)
 	}
 
 	if (batchmode != 0) {
+		if (rflag != 0) {
+			(void) fprintf(stderr, "Warning: -R flag may not work reliably for background jobs.\n");
+		}
 		/* List of files specified */
 		for (i=0; files[i] != NULL; i++) {
 			STRNCPY(ufile, files[i]);
@@ -377,6 +399,8 @@ main(int argc, char **argv)
 			}
 
 			result = SpoolX(
+				(batchmode < 3) ? NULL : stdout,
+				NULL,
 				"put",
 				ufilep, 	/* Remote file */
 				dstdir,		/* Remote CWD */
@@ -387,67 +411,93 @@ main(int argc, char **argv)
 				fi.port,
 				fi.user,
 				fi.pass,
+				fi.acct,
 				xtype,
 				rflag,
 				deleteflag,
 				fi.dataPortMode,
-				(time_t) 0	/* when: now */
+				precmd,
+				perfilecmd,
+				postcmd,
+				NULL,
+				NULL,
+				(time_t) 0,	/* when: now */
+				0
 			);
-			if (result == 0) {
+			if ((result == 0) && (batchmode < 3)) {
 				fprintf(stdout, "  + Spooled; sending remotely as %s/%s.\n", dstdir, ufilep);
 				spooled++;
 			}
 		}
-		if (spooled > 0) {
-			RunBatch(0, NULL);
-			DisposeWinsock(0);
+		if ((spooled > 0) || (batchmode >= 3)) {
+			if (batchmode == 1) {
+				RunBatch();
+			}
+			DisposeWinsock();
 			exit(kExitSuccess);
 		}
-		DisposeWinsock(0);
+		DisposeWinsock();
 		exit(kExitSpoolFailed);
 	}
 	
-	errstr = "could not open remote host";
 	es = kExitOpenTimedOut;
 	if ((result = FTPOpenHost(&fi)) < 0) {
 		(void) fprintf(stderr, "ncftpput: cannot open %s: %s.\n", fi.host, FTPStrError(result));
 		es = kExitOpenFailed;
-		DisposeWinsock(0);
+		DisposeWinsock();
 		exit((int) es);
 	}
+
 	if (fi.hasCLNT != kCommandNotAvailable)
 		(void) FTPCmd(&fi, "CLNT NcFTPPut %.5s %s", gVersion + 11, gOS);
+
 	if (Umask != NULL) {
-		errstr = "could not set umask on remote host";
 		result = FTPUmask(&fi, Umask);
 		if (result != 0)
 			FTPPerror(&fi, result, kErrUmaskFailed, "ncftpput", "could not set umask");
 	}
-	if (wantMkdir != 0) {
-		errstr = "could not mkdir on remote host";
-		result = FTPMkdir2(&fi, dstdir, kRecursiveYes, fi.startingWorkingDirectory);
-		if (result != 0)
-			FTPPerror(&fi, result, kErrMKDFailed, "ncftpput: Could not create directory", dstdir);
+
+	(void) AdditionalCmd(&fi, precmd, NULL);
+
+	if (dstdir != NULL) {
+		es = kExitChdirTimedOut;
+		if (wantMkdir != 0) {
+			result = FTPChdir3(&fi, dstdir, NULL, 0, kChdirFullPath|kChdirOneSubdirAtATime|kChdirAndMkdir);
+		} else {
+			result = FTPChdir3(&fi, dstdir, NULL, 0, kChdirFullPath|kChdirOneSubdirAtATime);
+		}
 	}
+
+	if (result != 0) {
+		FTPPerror(&fi, result, kErrCWDFailed, "ncftpput: Could not change to directory", dstdir);
+		(void) FTPCloseHost(&fi);
+		es = kExitChdirFailed;
+		DisposeWinsock();
+		exit((int) es);
+	}
+
 	if (result >= 0) {
-		errstr = "could not write to file on remote host";
 		es = kExitXferTimedOut;
 		(void) signal(SIGINT, Abort);
 		if (ftpcat == 0) {
-			if (Copy(&fi, dstdir, files, rflag, xtype, appendflag, (const char *) tmppfx, (const char *) tmpsfx, resumeflag, deleteflag) < 0)
+			if (Copy(&fi, "", files, rflag, xtype, appendflag, (const char *) tmppfx, (const char *) tmpsfx, resumeflag, deleteflag, perfilecmd) < 0)
 				es = kExitXferFailed;
 			else
 				es = kExitSuccess;
 		} else {
 			fi.progress = (FTPProgressMeterProc) 0;
-			if (FTPPutOneFile2(&fi, NULL, argv[optind + 1], xtype, STDIN_FILENO, appendflag, tmppfx, tmpsfx) < 0)
+			if (FTPPutOneFile2(&fi, NULL, argv[opt.ind + 1], xtype, STDIN_FILENO, appendflag, tmppfx, tmpsfx) < 0)
 				es = kExitXferFailed;
-			else
+			else {
 				es = kExitSuccess;
+				(void) AdditionalCmd(&fi, perfilecmd, argv[opt.ind + 1]);
+			}
 		}
 	}
+
+	(void) AdditionalCmd(&fi, postcmd, NULL);
 	
 	(void) FTPCloseHost(&fi);
-	DisposeWinsock(0);
+	DisposeWinsock();
 	exit((int) es);
 }	/* main */
