@@ -9,16 +9,35 @@
 #	include <config.h>
 #endif
 
-#if defined(AIX) || defined(_AIX)
+#if defined(AIX) || defined(_AIX) || defined(__HOS_AIX__)
 #	define _ALL_SOURCE 1
 #endif
 
-#if defined(HPUX) || defined(_HPUX_SOURCE)
-#	ifndef _XOPEN_SOURCE_EXTENDED
-#		define _XOPEN_SOURCE_EXTENDED 1
-#	endif
+#ifdef LINUX
+#	define _GNU_SOURCE 1
 #endif
 
+#if defined(DIGITAL_UNIX) || defined(TRU64UNIX)
+#	define _OSF_SOURCE 1
+#	define _XOPEN_SOURCE 500
+#	define _XOPEN_SOURCE_EXTENDED 1
+#endif
+
+#ifdef HPUX
+#	define _HPUX_SOURCE 1
+#	define _XOPEN_SOURCE 500
+#	define _XOPEN_SOURCE_EXTENDED 1
+#endif
+
+#ifdef IRIX
+#	define _SGI_SOURCE 1
+#endif
+
+#ifdef SOLARIS
+/* #define __EXTENSIONS__ 1 */
+#endif
+
+
 #ifdef HAVE_UNISTD_H
 #	include <unistd.h>
 #endif
@@ -35,6 +54,26 @@
 #	include <sys/utsname.h>
 #endif
 
+/* The only reason we need to include this junk, is because on some systems
+ * the function killchar() is actually a macro that uses definitions in
+ * termios.h.  Example:  #define killchar()      (__baset.c_cc[VKILL])
+ */
+
+#ifdef HAVE_TERMIOS_H
+#	include <termios.h>
+#else
+#	ifdef HAVE_TERMIO_H
+#		include <termio.h>
+#	else
+#		ifdef HAVE_SYS_IOCTL_H
+#			include <sys/ioctl.h>	/* For TIOxxxx constants. */
+#		endif
+#		ifdef HAVE_SGTTY_H
+#			include <sgtty.h>
+#		endif
+#	endif
+#endif /* !HAVE_TERMIOS_H */
+
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -58,6 +97,32 @@
 #ifdef HAVE_LOCALE_H
 #	include <locale.h>
 #endif
+#ifdef NEED_GETOPT_H
+#	include <getopt.h>
+#elif defined(NEED_GETOPT_EXTERN_DECLS)
+	extern int optind, opterr, optopt;
+	extern char *optarg;
+#endif
+
+#ifndef FOPEN_READ_TEXT
+#	define FOPEN_READ_TEXT "r"
+#	define FOPEN_WRITE_TEXT "w"
+#	define FOPEN_APPEND_TEXT "a"
+#endif
+#ifndef FOPEN_READ_BINARY
+#	define FOPEN_READ_BINARY "r"
+#	define FOPEN_WRITE_BINARY "w"
+#	define FOPEN_APPEND_BINARY "a"
+#endif
+
+#if defined(MACOSX) || defined(BSDOS)
+#	undef SIG_DFL
+#	undef SIG_IGN
+#	undef SIG_ERR
+#	define SIG_DFL         (void (*)(int))0
+#	define SIG_IGN         (void (*)(int))1
+#	define SIG_ERR         (void (*)(int))-1
+#endif
 
 #ifdef HAVE_NCURSES_H
 #	include <ncurses.h>
@@ -67,48 +132,62 @@
 #	endif
 #endif
 
-/* These next two sections are mostly for HP-UX. */
-#if defined(HAVE___GETMAXX) && !defined(HAVE_GETMAXX) && !defined(getmaxx)
-#	define HAVE_GETMAXX 1
-#	define getmaxx __getmaxx
-#endif
-#if defined(HAVE___GETMAXY) && !defined(HAVE_GETMAXY) && !defined(getmaxy)
-#	define HAVE_GETMAXY 1
-#	define getmaxy __getmaxy
-#endif
-#if defined(HAVE___GETMAXY) && defined(HAVE___GETMAXX) && !defined(HAVE_GETMAXYX) && !defined(getmaxyx)
-#	define HAVE_GETMAXYX 1
-#	define getmaxyx(__win,__y,__x) { WINDOW *__wi; __wi = __win; ((__y) = __getmaxy(__wi), (__x) = __getmaxx(__wi)); }
+/* These next three sections are mostly for HP-UX 10. */
+#if defined(HAVE___GETCURX) && defined(HAVE___GETCURY)
+#	ifndef HAVE_GETYX
+#		define HAVE_GETYX 1
+#	endif
+#	undef getyx
+#	define getyx(w,y,x) (y) = __getcury((w)); (x) = __getcurx((w))
 #endif
 
-#if defined(HAVE___GETBEGX) && !defined(HAVE_GETBEGX) && !defined(getbegx)
+#if defined(HAVE___GETMAXX) && !defined(HAVE_GETMAXX)
+#	define HAVE_GETMAXX 1
+#	undef getmaxx
+#	define getmaxx __getmaxx
+#endif
+#if defined(HAVE___GETMAXY) && !defined(HAVE_GETMAXY)
+#	define HAVE_GETMAXY 1
+#	undef getmaxy
+#	define getmaxy __getmaxy
+#endif
+#if defined(HAVE___GETMAXY) && defined(HAVE___GETMAXX) && !defined(HAVE_GETMAXYX)
+#	define HAVE_GETMAXYX 1
+#	undef getmaxyx
+#	define getmaxyx(w,y,x) (y) = __getmaxy((w)); (x) = __getmaxx((w))
+#endif
+
+#if defined(HAVE___GETBEGX) && !defined(HAVE_GETBEGX)
 #	define HAVE_GETBEGX 1
+#	undef getbegx
 #	define getbegx __getbegx
 #endif
-#if defined(HAVE___GETBEGY) && !defined(HAVE_GETBEGY) && !defined(getbegy)
+#if defined(HAVE___GETBEGY) && !defined(HAVE_GETBEGY)
 #	define HAVE_GETBEGY 1
+#	undef getbegy
 #	define getbegy __getbegy
 #endif
-#if defined(HAVE___GETBEGY) && defined(HAVE___GETBEGX) && !defined(HAVE_GETBEGYX) && !defined(getbegyx)
+#if defined(HAVE___GETBEGY) && defined(HAVE___GETBEGX) && !defined(HAVE_GETBEGYX)
 #	define HAVE_GETBEGYX 1
-#	define getbegyx(__win,__y,__x) { WINDOW *__wi; __wi = __win; ((__y) = __getbegy(__wi), (__x) = __getbegx(__wi)); }
+#	undef getbegyx
+#	define getbegyx(w,y,x) (y) = __getbegy((w)); (x) = __getbegx((w))
 #endif
 
 /* Otherwise, try accessing the structure directly. */
 #ifndef HAVE_GETMAXYX
 #	ifdef HAVE__MAXX
 #		ifndef getmaxyx
-#			define getmaxyx(w,y,x) y = w->_maxy;  x = w->_maxx;
+#			define getmaxyx(w,y,x) y = w->_maxy;  x = w->_maxx
 #		endif
 #		ifndef getbegyx
-#			define getbegyx(w,y,x) y = w->_begy;  x = w->_begx;
+#			define getbegyx(w,y,x) y = w->_begy;  x = w->_begx
 #		endif
 #	else
 #		ifndef getmaxyx
-#			define getmaxyx(w,y,x) y = w->maxy;  x = w->maxx;
+#			define getmaxyx(w,y,x) y = w->maxy;  x = w->maxx
 #		endif
 #		ifndef getbegyx
-#			define getbegyx(w,y,x) y = w->begy;  x = w->begx;
+#			define getbegyx(w,y,x) y = w->begy;  x = w->begx
 #		endif
 #	endif
 #endif
@@ -205,7 +284,11 @@
 #	define Open open
 #endif
 
-#if defined(HAVE_LONG_LONG) && defined(HAVE_STAT64) && defined(HAVE_STRUCT_STAT64)
+#if defined(WIN32) || defined(_WINDOWS)
+#	define Stat WinStat64
+#	define Lstat WinStat64
+#	define Fstat WinFStat64
+#elif defined(HAVE_LONG_LONG) && defined(HAVE_STAT64) && defined(HAVE_STRUCT_STAT64)
 #	define Stat stat64
 #	ifdef HAVE_FSTAT64
 #		define Fstat fstat64
@@ -239,5 +322,12 @@
 #	define Lseek(a,b,c) lseek(a, (off_t) b, c)
 #endif
 
+#if (defined(SOCKS)) && (SOCKS >= 5)
+#	ifdef HAVE_SOCKS_H
+#		include <socks.h>
+#	endif
+#endif	/* SOCKS */
+
 #include <Strn.h>			/* Library header. */
+#include <sio.h>			/* Because ../ncftp/util.c needs it. */
 #include <ncftp.h>			/* Mostly for utility routines it has. */
